=== modified file 'drizzled/cursor.cc'
--- drizzled/cursor.cc	2010-04-21 02:28:11 +0000
+++ drizzled/cursor.cc	2010-04-23 20:52:47 +0000
@@ -43,6 +43,7 @@
 #include "drizzled/message/table.pb.h"
 #include "drizzled/plugin/client.h"
 #include "drizzled/internal/my_sys.h"
+#include "drizzled/plugin/event.h"
 
 using namespace std;
 
@@ -1481,9 +1482,17 @@
   if (table->timestamp_field_type & TIMESTAMP_AUTO_SET_ON_INSERT)
     table->timestamp_field->set_time();
 
-  DRIZZLE_INSERT_ROW_START(table_share->getSchemaName(), table_share->getTableName());
+  DRIZZLE_INSERT_ROW_START(table->getSchemaName(), table_share->getTableName());
   setTransactionReadWrite();
-  error= doInsertRecord(buf);
+  
+  if (unlikely(plugin::Event::preWriteRowDo(table->in_use, table_share, buf)))
+    error= ER_EVENT_PLUGIN;
+  else
+  {
+    error= doInsertRecord(buf);
+    plugin::Event::postWriteRowDo(table->in_use, table_share, buf, error);
+  }
+ 
   DRIZZLE_INSERT_ROW_DONE(error);
 
   if (unlikely(error))
@@ -1510,7 +1519,14 @@
 
   DRIZZLE_UPDATE_ROW_START(table_share->getSchemaName(), table_share->getTableName());
   setTransactionReadWrite();
-  error= doUpdateRecord(old_data, new_data);
+  if (unlikely(plugin::Event::preUpdateRowDo(table->in_use, table_share, old_data, new_data)))
+    error= ER_EVENT_PLUGIN;
+  else
+  {
+    error= doUpdateRecord(old_data, new_data);
+    plugin::Event::postUpdateRowDo(table->in_use, table_share, old_data, new_data, error);
+  }
+
   DRIZZLE_UPDATE_ROW_DONE(error);
 
   if (unlikely(error))
@@ -1530,7 +1546,14 @@
 
   DRIZZLE_DELETE_ROW_START(table_share->getSchemaName(), table_share->getTableName());
   setTransactionReadWrite();
-  error= doDeleteRecord(buf);
+  if (unlikely(plugin::Event::preDeleteRowDo(table->in_use, table_share, buf)))
+    error= ER_EVENT_PLUGIN;
+  else
+  {
+    error= doDeleteRecord(buf);
+    plugin::Event::postDeleteRowDo(table->in_use, table_share, buf, error);
+  }
+
   DRIZZLE_DELETE_ROW_DONE(error);
 
   if (unlikely(error))

=== modified file 'drizzled/enum.h'
--- drizzled/enum.h	2010-04-20 18:23:07 +0000
+++ drizzled/enum.h	2010-04-23 20:52:47 +0000
@@ -1056,7 +1056,8 @@
   ER_ALTER_SCHEMA,
   ER_DROP_SCHEMA,
   ER_USE_SQL_BIG_RESULT,
-  ER_ERROR_LAST= ER_USE_SQL_BIG_RESULT
+  ER_EVENT_PLUGIN,
+  ER_ERROR_LAST= ER_EVENT_PLUGIN
 };
 
 enum drizzle_exit_codes {

=== modified file 'drizzled/include.am'
--- drizzled/include.am	2010-04-20 19:03:28 +0000
+++ drizzled/include.am	2010-04-23 20:52:47 +0000
@@ -296,6 +296,7 @@
 			      drizzled/plugin/context.h \
 			      drizzled/plugin/daemon.h \
 			      drizzled/plugin/error_message.h \
+			      drizzled/plugin/event.h \
 			      drizzled/plugin/function.h \
 			      drizzled/plugin/library.h \
 			      drizzled/plugin/listen.h \
@@ -668,6 +669,7 @@
 				 drizzled/plugin/authorization.cc \
 				 drizzled/plugin/client.cc \
 				 drizzled/plugin/error_message.cc \
+				 drizzled/plugin/event.cc  \
 				 drizzled/plugin/function.cc \
 				 drizzled/plugin/library.cc \
 				 drizzled/plugin/listen.cc \

=== modified file 'drizzled/plugin/storage_engine.cc'
--- drizzled/plugin/storage_engine.cc	2010-04-16 05:36:53 +0000
+++ drizzled/plugin/storage_engine.cc	2010-04-23 20:52:47 +0000
@@ -53,6 +53,7 @@
 #include "drizzled/db.h"
 
 #include <drizzled/table_proto.h>
+#include <drizzled/plugin/event.h>
 
 static bool shutdown_has_begun= false; // Once we put in the container for the vector/etc for engines this will go away.
 
@@ -98,9 +99,18 @@
 
 int StorageEngine::renameTable(Session &session, TableIdentifier &from, TableIdentifier &to)
 {
+  int error;
   setTransactionReadWrite(session);
 
-  return doRenameTable(session, from, to);
+  if (unlikely(plugin::Event::preRenameTableDo(&session, &from, &to)))
+    error= ER_EVENT_PLUGIN;
+  else
+  {
+    error =  doRenameTable(session, from, to);
+    plugin::Event::postRenameTableDo(&session, &from, &to, error);
+  }
+  
+  return error;
 }
 
 /**
@@ -464,7 +474,10 @@
   int error;
 
   engine.setTransactionReadWrite(session);
+  
+  plugin::Event::preDropTableDo(&session, &identifier);
   error= engine.doDropTable(session, identifier);
+  plugin::Event::postDropTableDo(&session, &identifier, error);
 
   return error;
 }

=== modified file 'drizzled/session.cc'
--- drizzled/session.cc	2010-04-20 00:04:22 +0000
+++ drizzled/session.cc	2010-04-23 20:52:47 +0000
@@ -50,6 +50,7 @@
 
 #include "plugin/myisam/myisam.h"
 #include "drizzled/internal/iocache.h"
+#include "drizzled/plugin/event.h"
 
 #include <fcntl.h>
 #include <algorithm>
@@ -189,7 +190,8 @@
   m_lip(NULL),
   cached_table(0),
   transaction_message(NULL),
-  statement_message(NULL)
+  statement_message(NULL),
+  session_event_observers(NULL)
 {
   memset(process_list_info, 0, PROCESS_LIST_WIDTH);
   client->setSession(this);
@@ -263,6 +265,8 @@
   thr_lock_owner_init(&main_lock_id, &lock_info);
 
   m_internal_handler= NULL;
+  
+  plugin::Event::registerSessionEventsDo(this); 
 }
 
 void Session::free_items()
@@ -368,6 +372,7 @@
   pthread_setspecific(THR_Session,  0);
 
   plugin::Logging::postEndDo(this);
+  plugin::Event::deregisterSessionEventsDo(this); 
 
   /* Ensure that no one is using Session */
   pthread_mutex_unlock(&LOCK_delete);

=== modified file 'drizzled/session.h'
--- drizzled/session.h	2010-04-19 21:42:39 +0000
+++ drizzled/session.h	2010-04-23 20:52:47 +0000
@@ -59,7 +59,10 @@
 {
 class Client;
 class Scheduler;
+class SessionEventObservers;
+class SchemaEventObservers;
 }
+
 namespace message
 {
 class Transaction;
@@ -1252,7 +1255,50 @@
   /** Pointers to memory managed by the ReplicationServices component */
   message::Transaction *transaction_message;
   message::Statement *statement_message;
-  /** Microsecond timestamp of when Session connected */
+  plugin::SessionEventObservers *session_event_observers;
+  
+  /* Schema observers are mapped to databases. */
+  std::map<std::string, plugin::SchemaEventObservers *> schema_event_observers;
+
+ 
+public:
+  plugin::SessionEventObservers *getSessionObservers() 
+  { 
+    return session_event_observers;
+  }
+  
+  void setSessionObservers(plugin::SessionEventObservers *observers) 
+  { 
+    session_event_observers= observers;
+  }
+  
+  /* For schema event observers there is one set of observers per database. */
+  plugin::SchemaEventObservers *getSchemaObservers(const std::string *db_name) 
+  { 
+    std::map<std::string, plugin::SchemaEventObservers *>::iterator it;
+    
+    it= schema_event_observers.find(*db_name);
+    if (it == schema_event_observers.end())
+      return NULL;
+      
+    return it->second;
+  }
+  
+  void setSchemaObservers(const std::string *db_name, plugin::SchemaEventObservers *observers) 
+  { 
+    std::map<std::string, plugin::SchemaEventObservers *>::iterator it;
+
+    it= schema_event_observers.find(*db_name);
+    if (it != schema_event_observers.end())
+      schema_event_observers.erase(it);;
+
+    if (observers)
+      schema_event_observers[*db_name] = observers;
+  }
+  
+  
+ private:
+ /** Microsecond timestamp of when Session connected */
   uint64_t connect_microseconds;
   const char *proc_info;
 

=== modified file 'drizzled/statement/create_schema.cc'
--- drizzled/statement/create_schema.cc	2010-03-31 05:53:34 +0000
+++ drizzled/statement/create_schema.cc	2010-04-23 20:52:47 +0000
@@ -23,6 +23,7 @@
 #include <drizzled/session.h>
 #include <drizzled/statement/create_schema.h>
 #include <drizzled/db.h>
+#include <drizzled/plugin/event.h>
 
 #include <string>
 
@@ -51,7 +52,15 @@
     schema_message.set_collation(default_charset_info->name);
   }
 
-  bool res= mysql_create_db(session, schema_message, is_if_not_exists);
+  bool res = false;
+  if (unlikely(plugin::Event::preCreateDatabaseDo(session, schema_identifier.getSQLPath().c_str())))
+    my_error(ER_EVENT_PLUGIN, MYF(0), schema_identifier.getSQLPath().c_str());
+  else
+  {
+    res= mysql_create_db(session, schema_message, is_if_not_exists);
+    plugin::Event::postCreateDatabaseDo(session, schema_identifier.getSQLPath().c_str(), res);
+  }
+
   return not res;
 }
 

=== modified file 'drizzled/statement/drop_schema.cc'
--- drizzled/statement/drop_schema.cc	2010-03-31 05:53:34 +0000
+++ drizzled/statement/drop_schema.cc	2010-04-23 20:52:47 +0000
@@ -23,6 +23,7 @@
 #include <drizzled/session.h>
 #include <drizzled/statement/drop_schema.h>
 #include <drizzled/db.h>
+#include <drizzled/plugin/event.h>
 
 #include <string>
 
@@ -50,7 +51,15 @@
         MYF(0));
     return true;
   }
-  bool res= mysql_rm_db(session, schema_identifier, drop_if_exists);
+  
+  bool res = true;
+  if (unlikely(plugin::Event::preDropDatabaseDo(session, schema_identifier.getSQLPath().c_str())))
+    my_error(ER_EVENT_PLUGIN, MYF(0), schema_identifier.getSQLPath().c_str());
+  else
+  {
+    res= mysql_rm_db(session, schema_identifier, drop_if_exists);
+    plugin::Event::postDropDatabaseDo(session, schema_identifier.getSQLPath().c_str(), res);
+  }
 
   return res;
 }

=== modified file 'drizzled/table_share.cc'
--- drizzled/table_share.cc	2010-04-16 05:36:53 +0000
+++ drizzled/table_share.cc	2010-04-23 20:52:47 +0000
@@ -35,6 +35,7 @@
 #include "drizzled/sql_base.h"
 #include "drizzled/pthread_globals.h"
 #include "drizzled/internal/my_pthread.h"
+#include "drizzled/plugin/event.h"
 
 using namespace std;
 
@@ -108,6 +109,9 @@
   {
     const string key_string(share->table_cache_key.str,
                             share->table_cache_key.length);
+                            
+    plugin::Event::deregisterTableEventsDo(share);
+   
     TableDefinitionCache::iterator iter= table_def_cache.find(key_string);
     if (iter != table_def_cache.end())
     {
@@ -131,6 +135,7 @@
     if (share->ref_count == 0)
     {
       pthread_mutex_lock(&share->mutex);
+      plugin::Event::deregisterTableEventsDo(share);
       share->free_table_share();
       table_def_cache.erase(key_string);
     }
@@ -233,6 +238,9 @@
     return NULL;
   }
   share->ref_count++;				// Mark in use
+  
+  plugin::Event::registerTableEventsDo(share);
+  
   (void) pthread_mutex_unlock(&share->mutex);
 
   return share;

=== modified file 'drizzled/table_share.h'
--- drizzled/table_share.h	2010-04-19 19:23:14 +0000
+++ drizzled/table_share.h	2010-04-23 20:52:47 +0000
@@ -45,6 +45,11 @@
 const static std::string INTERNAL_STRING("INTERNAL");
 const static std::string FUNCTION_STRING("FUNCTION");
 
+namespace plugin
+{
+class TableEventObservers;
+}
+
 class TableShare
 {
 public:
@@ -101,7 +106,8 @@
     replace_with_name_lock(false),
     waiting_on_cond(false),
     keys_in_use(0),
-    keys_for_keyread(0)
+    keys_for_keyread(0),
+    event_observers(NULL)
   {
     init();
   }
@@ -162,7 +168,8 @@
     replace_with_name_lock(false),
     waiting_on_cond(false),
     keys_in_use(0),
-    keys_for_keyread(0)
+    keys_for_keyread(0),
+    event_observers(NULL)
   {
     init(key, key_length, new_table_name, new_path);
   }
@@ -409,6 +416,23 @@
   key_map keys_in_use;
   key_map keys_for_keyread;
 
+  /* 
+    event_observers is a class containing all the event plugins that have 
+    registered an interest in this table.
+  */
+  private:
+  plugin::TableEventObservers *event_observers;
+  public:
+  plugin::TableEventObservers *getTableObservers() 
+  { 
+    return event_observers;
+  }
+  
+  void setTableObservers(plugin::TableEventObservers *observers) 
+  { 
+    event_observers= observers;
+  }
+  
   /*
     Set share's table cache key and update its db and table name appropriately.
 

